<?php
/**
 * Plugin Name:         RS Scratchpad
 * Description:         Create a private post type for drafting, jotting, brainstorming -- without having to decide immediately which post type it should be.
 * Plugin URI:          https://plugins.seindal.dk/plugins/rs-scratchpad/
 * Update URI:          https://plugins.seindal.dk
 * Author:              René Seindal
 * Author URI:          https://plugins.seindal.dk/
 * Donate link:         https://mypos.com/@historywalks
 * Requires Plugins:    rs-base-plugin
 * License:             GPL v2 or later
 * License URI:         https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:         rs-scratchpad
 * Domain Path:         /languages
 * Requires PHP:        7.4
 * Requires at least:   5.0
 * Version:             0.6.6
 **/

namespace ReneSeindal;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

add_action( 'plugins_loaded', function() {
    class RS_Scratchpad extends PluginBase {
        protected $plugin_file = __FILE__;

        use PluginBaseSettings;


        /************************************************************************
         *
         * Registers the `scratchpad` post type.
         *
         ************************************************************************/

        function do_init_action_scratchpad_post_type() {
            $supports = [
                'author',
                'comments',
                'custom-fields',
                'editor',
                'excerpt',
                'page-attributes',
                'revisions',
                'thumbnail',
                'title',
            ];

            register_post_type(
                'scratchpad',
                [
                    'labels'                => [
                        'name'                  => __( 'Scratchpads', 'rs-scratchpad' ),
                        'singular_name'         => __( 'Scratchpad', 'rs-scratchpad' ),
                        'all_items'             => __( 'All Scratchpads', 'rs-scratchpad' ),
                        'archives'              => __( 'Scratchpad Archives', 'rs-scratchpad' ),
                        'attributes'            => __( 'Scratchpad Attributes', 'rs-scratchpad' ),
                        'insert_into_item'      => __( 'Insert into Scratchpad', 'rs-scratchpad' ),
                        'uploaded_to_this_item' => __( 'Uploaded to this Scratchpad', 'rs-scratchpad' ),
                        'featured_image'        => _x( 'Featured Image', 'scratchpad', 'rs-scratchpad' ),
                        'set_featured_image'    => _x( 'Set featured image', 'scratchpad', 'rs-scratchpad' ),
                        'remove_featured_image' => _x( 'Remove featured image', 'scratchpad', 'rs-scratchpad' ),
                        'use_featured_image'    => _x( 'Use as featured image', 'scratchpad', 'rs-scratchpad' ),
                        'filter_items_list'     => __( 'Filter Scratchpads list', 'rs-scratchpad' ),
                        'items_list_navigation' => __( 'Scratchpads list navigation', 'rs-scratchpad' ),
                        'items_list'            => __( 'Scratchpads list', 'rs-scratchpad' ),
                        'new_item'              => __( 'New Scratchpad', 'rs-scratchpad' ),
                        'add_new'               => __( 'Add New', 'rs-scratchpad' ),
                        'add_new_item'          => __( 'Add New Scratchpad', 'rs-scratchpad' ),
                        'edit_item'             => __( 'Edit Scratchpad', 'rs-scratchpad' ),
                        'view_item'             => __( 'View Scratchpad', 'rs-scratchpad' ),
                        'view_items'            => __( 'View Scratchpads', 'rs-scratchpad' ),
                        'search_items'          => __( 'Search Scratchpads', 'rs-scratchpad' ),
                        'not_found'             => __( 'No Scratchpads found', 'rs-scratchpad' ),
                        'not_found_in_trash'    => __( 'No Scratchpads found in trash', 'rs-scratchpad' ),
                        'parent_item_colon'     => __( 'Parent Scratchpad:', 'rs-scratchpad' ),
                        'menu_name'             => __( 'Scratchpads', 'rs-scratchpad' ),
                    ],
                    'public'                => defined( 'WP_CLI' ) && WP_CLI,
                    'hierarchical'          => false,
                    'show_ui'               => true,
                    'show_in_nav_menus'     => true,
                    'supports'              => $supports,
                    'has_archive'           => true,
                    'rewrite'               => true,
                    'query_var'             => true,
                    'menu_position'         => null,
                    'menu_icon'             => 'dashicons-lightbulb',
                    'show_in_rest'          => true,
                    'rest_base'             => 'scratchpad',
                    'rest_controller_class' => 'WP_REST_Posts_Controller',
                ]
            );

            register_post_meta(
                'scratchpad',
                'footnotes',
                [
                    'show_in_rest'      => true,
                    'single'            => true,
                    'type'              => 'string',
                    'revisions_enabled' => true,
                ]
            );

        }

        /**
         * Sets the post updated messages for the `scratchpad` post type.
         *
         * @param  array $messages Post updated messages.
         * @return array Messages for the `scratchpad` post type.
         */

        function do_post_updated_messages_filter_scratchpad( $messages ) {
            global $post;

            $permalink = get_permalink( $post );

            $messages['scratchpad'] = [
                0  => '', // Unused. Messages start at index 1.
                /* translators: %s: post permalink */
                1  => sprintf( __( 'Scratchpad updated. <a target="_blank" href="%s">View Scratchpad</a>', 'rs-scratchpad' ), esc_url( $permalink ) ),
                2  => __( 'Custom field updated.', 'rs-scratchpad' ),
                3  => __( 'Custom field deleted.', 'rs-scratchpad' ),
                4  => __( 'Scratchpad updated.', 'rs-scratchpad' ),
                /* translators: %s: date and time of the revision */
                5  => isset( $_GET['revision'] ) ? sprintf( __( 'Scratchpad restored to revision from %s', 'rs-scratchpad' ), wp_post_revision_title( (int) $_GET['revision'], false ) ) : false, // phpcs:ignore WordPress.Security.NonceVerification.Recommended
                /* translators: %s: post permalink */
                6  => sprintf( __( 'Scratchpad published. <a href="%s">View Scratchpad</a>', 'rs-scratchpad' ), esc_url( $permalink ) ),
                7  => __( 'Scratchpad saved.', 'rs-scratchpad' ),
                /* translators: %s: post permalink */
                8  => sprintf( __( 'Scratchpad submitted. <a target="_blank" href="%s">Preview Scratchpad</a>', 'rs-scratchpad' ), esc_url( add_query_arg( 'preview', 'true', $permalink ) ) ),
                /* translators: 1: Publish box date format, see https://secure.php.net/date 2: Post permalink */
                9  => sprintf( __( 'Scratchpad scheduled for: <strong>%1$s</strong>. <a target="_blank" href="%2$s">Preview Scratchpad</a>', 'rs-scratchpad' ), date_i18n( __( 'M j, Y @ G:i', 'rs-scratchpad' ), strtotime( $post->post_date ) ), esc_url( $permalink ) ),
                /* translators: %s: post permalink */
                10 => sprintf( __( 'Scratchpad draft updated. <a target="_blank" href="%s">Preview Scratchpad</a>', 'rs-scratchpad' ), esc_url( add_query_arg( 'preview', 'true', $permalink ) ) ),
            ];

            return $messages;
        }

        /**
         * Sets the bulk post updated messages for the `scratchpad` post type.
         *
         * @param  array $bulk_messages Arrays of messages, each keyed by the corresponding post type. Messages are
         *                              keyed with 'updated', 'locked', 'deleted', 'trashed', and 'untrashed'.
         * @param  int[] $bulk_counts   Array of item counts for each message, used to build internationalized strings.
         * @return array Bulk messages for the `scratchpad` post type.
         */


        function do_bulk_post_updated_messages_filter_scratchpad( $bulk_messages, $bulk_counts ) {
            global $post;

            $bulk_messages['scratchpad'] = [
                /* translators: %s: Number of Scratchpads. */
                'updated'   => _n( '%s Scratchpad updated.', '%s Scratchpads updated.', $bulk_counts['updated'], 'rs-scratchpad' ),
                'locked'    => ( 1 === $bulk_counts['locked'] ) ? __( '1 Scratchpad not updated, somebody is editing it.', 'rs-scratchpad' ) :
                /* translators: %s: Number of Scratchpads. */
                _n( '%s Scratchpad not updated, somebody is editing it.', '%s Scratchpads not updated, somebody is editing them.', $bulk_counts['locked'], 'rs-scratchpad' ),
                /* translators: %s: Number of Scratchpads. */
                'deleted'   => _n( '%s Scratchpad permanently deleted.', '%s Scratchpads permanently deleted.', $bulk_counts['deleted'], 'rs-scratchpad' ),
                /* translators: %s: Number of Scratchpads. */
                'trashed'   => _n( '%s Scratchpad moved to the Trash.', '%s Scratchpads moved to the Trash.', $bulk_counts['trashed'], 'rs-scratchpad' ),
                /* translators: %s: Number of Scratchpads. */
                'untrashed' => _n( '%s Scratchpad restored from the Trash.', '%s Scratchpads restored from the Trash.', $bulk_counts['untrashed'], 'rs-scratchpad' ),
            ];

            return $bulk_messages;
        }



        /*************************************************************
         *
         * Add links to post row
         *
         *************************************************************/

        function do_post_row_actions_filter_duplicate_post( $actions, $post ) {
            if ( 'scratchpad' == $post->post_type ) {
                $post_types = $this->get_option( 'post_types' ) ?? [];

                foreach ( $post_types as $post_type ) {
                    $obj = get_post_type_object( $post_type );
                    if ( $obj ) {
                        $label = sprintf( __( '&rarr; %s', 'rs-scratchpad' ), $obj->labels->singular_name );
                        $action = $this->post_type_action( $post_type );
                        $nonce = sprintf( 'scratchpad_to_%s_%d', $post_type, $post->ID );

                        $url = $this->build_admin_url( 'admin-post.php', [
                            'action' => $action,
                            'post' => $post->ID,
                            'post_type' => $post_type,
                            '_wpnonce' => wp_create_nonce( $nonce ),
                        ] );

                        $link = sprintf( '<a href="%1$s">%2$s</a>', esc_url( $url ), esc_html( $label ) );

                        $actions[ $action ] = $link;
                    }
                }
            }

            return $actions;
        }

        function do_page_row_actions_filter_duplicate_post( $actions, $post ) {
            return $this->do_post_row_actions_filter_duplicate_post( $actions, $post );
        }

        private function post_type_action( $post_type ) {
            return 'scratchpad_' . strtr( sanitize_key( $post_type ), '-', '_' );
        }

        /*************************************************************
         *
         * Setup the action handlers
         *
         *************************************************************/

        function do_init_action_setup_handlers() {
            $post_types = $this->get_option( 'post_types' ) ?? [];

            foreach ( $post_types as $post_type ) {
                $hook =  'admin_post_' . $this->post_type_action( $post_type );
                add_action( $hook, [ $this, 'convert_to_post_type_handler' ] );
            }
        }


        function convert_to_post_type_handler() {
            error_log( print_r( 'convert_to_post_type', true ) );

            if ( ! $this->convert_to_post_type() )
                wp_die( __( 'Illegal request', 'rs-scratchpad' ) );
        }

        function convert_to_post_type() {
            if ( ! isset( $_REQUEST['post'] ) ) return;
            if ( ! isset( $_REQUEST['post_type'] ) ) return;
            if ( ! isset( $_REQUEST['_wpnonce'] ) ) return;

            // Find the post
            $post_id = absint( $_REQUEST['post'] );
            if ( $post_id <= 0 ) return;

            $post = get_post( $post_id );
            if ( ! $post ) return;

            // Find the post type
            $post_type = get_post_type_object( sanitize_key( $_REQUEST['post_type'] ) );
            if ( !$post_type ) return;

            // Check the nonce
            $nonce = sprintf( 'scratchpad_to_%s_%d', $post_type->name, $post_id );
            if ( ! wp_verify_nonce( $_REQUEST['_wpnonce'], $nonce ) )
                return;

            $id = wp_update_post( [
                'ID' => $post->ID,
                'post_type' => $post_type->name,
                'post_status' => 'draft',
            ], true );

            if( is_wp_error( $id ) ) {
                wp_die( $id );
                exit;
            }

            $url = $this->build_admin_url( 'post.php', [ 'post' => $post->ID, 'action' => 'edit' ] );
            wp_redirect( $url );
            exit;
        }



        /************************************************************************
         *
         *	Connect taxonomies
         *
         ************************************************************************/

        function do_wp_loaded_action() {
            $taxonomies = $this->get_option( 'taxonomies' );

            if ( $taxonomies )
                foreach ( $taxonomies as $taxonomy )
                    register_taxonomy_for_object_type( $taxonomy, 'scratchpad' );
        }

        // This is for the Issue Taxonomy plugin - but no-op if not used
        function do_issues_taxonomy_post_types_filter( $post_types ) {
            $post_types[] = 'scratchpad';
            return $post_types;
        }

        // This is for the Word count plugin - but no-op if not used
        function do_word_count_post_types_filter( $post_types ) {
            $post_types[] = 'scratchpad';
            return $post_types;
        }


        /************************************************************************
         *
         *	Connect to dashboard widgets -- count both published,
         *	scheduled and drafts as this is not a public post types
         *
         ************************************************************************/

        function do_rs_dashboard_widgets_everywhere_filter( $post_types ) {
            $post_types[] = 'scratchpad';
            return $post_types;
        }

        function do_rs_dashboard_widgets_post_count_filter( $num, $post_type, $post_counts ) {
            return $post_counts->publish + $post_counts->future + $post_counts->draft;
        }

        /************************************************************************
         *
         *	Settings
         *
         ************************************************************************/

        function do_admin_menu_action() {
            $this->settings_add_menu(
                __( 'RS Scratchpad', 'rs-scratchpad'),
                __( 'RS Scratchpad', 'rs-scratchpad' )
            );
        }

        function settings_define_sections_and_fields() {
            $section = $this->settings_add_section(
                'section_taxonomies',
                __( 'Taxonomies', 'rs-scratchpad' ),
                __( 'Select the taxonomies you want to apply to your scratchpads.', 'rs-scratchpad' )
            );

            $this->settings_build_taxonomies_menu(
                'taxonomies', $section,
                __( 'Taxonomies', 'rs-scratchpad' ),
                [ 'public' => true, 'show_ui' => true, ],
                [],
                [
                    'rs_scratchpad_taxonomies',
                    'rs_custom_taxonomies',
                ]
            );

            // Post types

            $section = $this->settings_add_section(
                'section_post_types',
                __( 'Post-types', 'rs-scratchpad' ),
                __( 'Select the post-types you want to convert scratchpads to', 'rs-scratchpad' )
            );

            $this->settings_build_post_types_menu(
                'post_types', $section,
                __( 'Post-types', 'rs-scratchpad' ),
                [ 'public' => true ],
                [],
                [
                    'rs_scratchpad_post_types',
                    'rs_custom_post_types',
                ]
            );
        }

    }

    new RS_Scratchpad();
} );
